__doc__ = """\
Project:MOFS 
Start:  Sun Jul 10 04:11:21 MSK 2000
Author: Andrey Orlov  (c) 2002
Version:$Id: Image.py,v 1.36 2002/11/24 19:41:40 cray Exp $

Title:
        MOFS / Image Modified
        
Description :
    
        Some enhancement for tag function :
        New properties added, Check on allowed margin 
        of image size added.

Licence:
        GPL
        
Warranty:
        There is no warranty of any kind, either expressed
        or implied 
        
TODO:        
        Nothing

"""
__version__ = "$Revision: 1.36 $"
# $Source: /var/lib/cvs/IIGFS/zope/Products/MOFS/Image.py,v $

from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB
from Globals import default__class_init__

import OFS.Image
import string
import PropertyManager
from libIIGFS import *
from AccessControl.SecurityManagement import getSecurityManager

import l10n
_ = l10n.getL10n_decoder(__file__,'MOFS')
from l10n  import HTMLFile,DTMLFile

ImageLengthBlocked = 'ImageLengthBlocked'
ImageSizeBlocked = 'ImageSizeBlocked'


class Image (PropertyManager.PropertyManager,OFS.Image.Image) :
    """  """
        
    meta_type = 'Image'
    align  = 0
    border = 0
    alt    = ''
    ref    = ''

    manage_options = ( # what management options are there?
        {'label': _('Upload'),       'action': 'manage_main', 'help':('IIGFS','Image_Edit.stx') },
        {'label': _('Properties'),           'action': 'manage_propertiesForm', 'help':('IIGFS','Image_Properties.stx') }, 
    ) 
# + OFS.Image.Image.manage_options

    align_values = (('center',_('Center')),('right',_('Right')),('left',_('Left')))

# OFS.Image.Image._properties +
    _properties =  (
        {'id':'title',  'name':_('Title'),             'type': 'string',       'mode':'w'},
        {'id':'content_type', 'name':_('Content type'), 'type': 'string',       'mode':'w'},
        {'id':'height', 'name':_('Height'),                'type': 'string',       'mode':'w'},
        {'id':'width',  'name':_('Width'),                'type': 'string',       'mode':'w'},
        {'id':'align',  'name':_('Alignment'),          'type': 'selection',    'mode':'w',     'select_variable': 'align_values'},
        {'id':'border', 'name':_('Existence of a border'),       'type': 'boolean',      'mode':'w'},
        {'id':'alt',    'name':_('Alternative text'),  'type': 'string',       'mode':'w'},
        {'id':'ref',    'name':_('Link under the image'),       'type': 'string',       'mode':'w'},
        {'id':'order',  'name':_('number'),      'type': 'int',          'mode':'w'},
        {'id':'hidden',  'name':_('Hidden'),                     'type': 'boolean',      'mode':'w'},
    ) 

    __ac_permissions__ = (
        ('View IIGFS managment',    ('manage_propertiesForm','manage_propertyTypeForm')),
        ('Change IIGFS',            ('manage_edit','manage_editForm','manage_upload',
                                'manage_addProperty','manage_editProperties','manage_delProperties',
                                'manage_changeProperties','manage_changePropertyTypes',)),
        ('View IIGFS',              ('tag',) ),
        ('Allow Unlimited Image',              () ),
        ) 

    manage_editForm  =DTMLFile('dtml/imageEdit',globals(), Kind=_('Image'),kind=_('image'))
    manage_editForm._setName('manage_editForm')
    manage=manage_main=manage_editForm
    manage_uploadForm=manage_editForm
 
    def __init__(self, id, title='',content_type='',precondition='', alt='', border='', align='left',  ref='') : 
        """initialise a new instance of Image"""
        OFS.Image.Image.__init__(self,id,title,content_type,precondition)
        self.alt = alt
        self.border = border
        self.align = align
        self.ref = self.type_url_tunning(ref)
#        self.check_borders()

#    def index_html(self, REQUEST, RESPONSE) :
#       """    """
#       res = OFS.Image.Image.index_html(self, REQUEST, RESPONSE)
#       if not self.precondition :
#               self.response_cache_headers(REQUEST)
#       return res

    def manage_upload(self,file,REQUEST=None) :
        """    """
        res = OFS.Image.Image.manage_upload(self,file,REQUEST)
        self.check_borders()
        return res

    def check_borders(self) :
        res = 0
        if not getSecurityManager().checkPermission('Allow Unlimited Image', self) :
            try :
                f = self.cfg
            except AttributeError :
                f = self.__getattr__

            hin = f('image_min_height',0)
            hax = f('image_max_height',150)
            win = f('image_min_width',0)
            wax = f('image_max_width',150)
            sax = f('image_max_size',32000)
            ert = f('image_except_ratio',3)

            try :
                try :
                        h = float(self.height)
                        w = float(self.width)
                except ValueError :
                        return 0

                ratio = h / w
    
                if h/hax > ert or hin/h > ert :
                    raise ImageSizeBlocked,_('Image height is %u, available interval [%u,%u] exceeded') % (int(h),int(hin*ert),int(hax*ert))
                if w/wax > ert or win/w > ert :
                    raise ImageSizeBlocked,_('Image width is %u, available interval [%u,%u] exceeded') % (int(w),int(win*ert),int(wax*ert))
    
                if self.size > sax :
                    raise ImageLengthBlocked,_('Image length is %u bytes, maximum available length (%u) exceeded') % (int(self.size),int(sax))
        
                if hax < h :
                    h = hax
                    w = int(h/ratio)
                    res = 1

                if wax < w :
                    w = wax
                    h = int(ratio*w)
                    res = 1
    
                if hin > h :
                    h = hin
                    w = int(h/ratio)
                    res = 1
            
                if win > w :
                    w = win
                    h = int(ratio*w)
                    res = 1
                
                self.height = str(int(h))
                self.width = str(int(w))
            except ZeroDivisionError :
                pass
        return res

    def manage_edit(self, title, content_type, ref, precondition='', filedata=None, REQUEST=None):
        """
        Changes the title and content type attributes of the File or Image,
           URL   .
        """

        self.ref=self.type_url_tunning(ref)

        res = OFS.Image.Image.manage_edit(self, title, content_type, precondition, filedata, REQUEST)
        return res

    def manage_editProperties(self, REQUEST) :
        """       """
        res = PropertyManager.PropertyManager.manage_editProperties(self, REQUEST)
        if self.check_borders() :
                return self.manage_propertiesForm(self,REQUEST, manage_tabs_message='Saved changes, but image scaled')
        return res

    def tag(self,height=None, width=None, alt=None, scale=0, xscale=0, yscale=0, css_class=None, border=None, ref=None, align=None, target=None, **args) :
        if border is None :     
            if self.border :
                border = '1'
            else :
               border = '0'
        else :
            if border :
               border = '1'
            else :
               border = '0'

        if ref is None :        ref = self.ref
        if align is None :      align = self.align
        if alt is None :
            if self.alt :
                alt = self.alt

        if target is None and hasattr(self,'reftarget') and self.reftarget :
            target = self.reftarget

#       s = OFS.Image.Image.tag(self, height, width, alt, scale, xscale, yscale, css_class, border=border, align=align,args) 
        args['border'] = border
        if align != 'center' :
            args['align']  = align

        s = apply(OFS.Image.Image.tag,(self, height, width, alt, scale, xscale, yscale, css_class),args)


        if ref :
            if target :
                s = '<a href="%s" target="%s">%s</a>' % (ref,target,s)
            s = '<a href="%s">%s</a>' % (ref,s)

        if align == 'center' :
            s = "<center>" + s + "</center>"

        return s
default__class_init__ (Image)
