__doc__ = """\
Project:MOFS 
Start:  Sun Jul 10 04:11:21 MSK 2000
Author: Andrey Orlov  (c) 2002
Version:$Id: PropertyManager.py,v 1.17 2002/11/24 19:50:09 cray Exp $

Title:
        MOFS / Property Manager Modified
        
Description :
    
    
        Property manager are to used as mix-in for objects with some
        property sheet, editable "Through The Web".
    
            - Support of localized properties added : property have now
              title, not only id;
              
            - Support for localized properties with meta type "select" and
              kind of "select" : values source now return pairs (value, title),
              but not only value;
              
            - Support for fixed properties : It kind property can be created
              only via special interface;

Licence:
        GPL
        
Warranty:
        There is no warranty of any kind, either expressed
        or implied 
        
TODO:        
        Nothing

"""
__version__ = "$Revision: 1.17 $"
# $Source: /var/lib/cvs/IIGFS/zope/Products/MOFS/PropertyManager.py,v $

import OFS.PropertyManager
from Globals import default__class_init__
import l10n
_ = l10n.getL10n_decoder(__file__,'MOFS')
from l10n import DTMLFile

class PropertyManager(OFS.PropertyManager.PropertyManager) :

    manage_options = ( # what management options are there?
        {'label': _('Properties'),           'action': 'manage_propertiesForm', 'help':('IIGFS','Properties.stx')}, 
    ) 


    __ac_permissions__=(
        ('Manage properties', ('manage_addProperty',
                               'manage_editProperties',
                               'manage_delProperties',
                               'manage_changeProperties',
                               'manage_propertiesForm',
                               'manage_propertyTypeForm',
                               'manage_changePropertyTypes',
                               'manage_propertyFixed',
                               'manage_addFixedProperty',
                               'fixedpropertyRecords',
                               )),
    )      


    manage_propertiesForm=DTMLFile('dtml/properties', globals(),
                                   property_extensible_schema__=1)

    manage_propertyTypeForm=DTMLFile('dtml/propertyType', globals())
    manage_propertyFixed=DTMLFile('dtml/propertyFixed', globals())

#    def manage_editProperties(self, REQUEST):
#        """   .
#        The purpose of this method is to change all property values,
#        even those not listed in REQUEST; otherwise checkboxes that
#        get turned off will be ignored.  Use manage_changeProperties()
#        instead for most situations.
#        """
#        for prop in self._propertyMap():
#            name=prop['id']
#            if 'w' in prop.get('mode', 'wd'):
#                value=REQUEST.get(name, '')
#                self._updateProperty(name, value, call)
#  
#       if REQUEST:
#            message="Saved changes."
#            return self.manage_propertiesForm(self,REQUEST,
#                                              manage_tabs_message=message)


    def fixedpropertyMaps(self) :
        try :
            d = self.FixedProperties
        except AttributeError :
            return []
        else :
            return filter(lambda x : not hasattr(self.aq_base,x['id']), d.fixedpropertyMaps())

    def hasFixedProperty(self,id) :
        try :
            d = self.FixedProperties
        except AttributeError :
            return 0
        else :
            return d.hasFixedProperty(id)

    def inlineEditItemForm(self,field) :
        for item in self._properties :
            if item['id'] == field :
                res = item.copy()
                res['value'] = getattr(self,field)
                res['field'] = 'field.%s:records:%s' % (res['id'],res['type'])
                return res
        
        return { 'id' : '', 'type':'undefined' }

        
#    def inlineEditItem() :
#        return

    def manage_addFixedProperty(self,ids,REQUEST=None) :
        """    """
        
        try :
            fp = self.FixedProperties
        except AttributeError :
            raise 
        else :
            for id in ids :
                d = fp.getFixedProperty(id)
                ret = OFS.PropertyManager.PropertyManager.manage_addProperty(self, d['id'],d['value'],d['type'], REQUEST)
                self._properties[-1]['name'] = str(d['name'])
        
        if REQUEST is not None:
            return self.manage_propertiesForm(self, REQUEST)

    def all_parent_dtml(self) :
        return map(lambda x : (x.getId(),x.title_and_id()), self.superValues('DTML Method'))

    def manage_addProperty(self, id, value, type, name=None, REQUEST=None):
        """  .
            .  
           ,   . 
            ,   ."""


        try :
            d = self.FixedProperties
        except AttributeError :
            pass
        else :
            if d.hasFixedProperty(id) :
                raise 'ForbiddenName',_('Forbidden property name')
                
        ret = OFS.PropertyManager.PropertyManager.manage_addProperty(self, id, value, type, REQUEST)

        if name is not None :
            self._properties[-1]['name'] = str(name)

        return ret



    def manage_changePropertyTypes(self, old_ids, props, REQUEST=None):
        """Replace one set of properties with another

        Delete all properties that have ids in old_ids, then add a
        property for each item in props.  Each item has a new_id,
        new_value, and new_type.  The type of new_value should match
        new_type.
        """
        err = self.manage_delProperties(old_ids)
        if err:
            if REQUEST is not None:
                return err
            return
        for prop in props:
            self._setProperty(prop.new_id, prop.new_value, prop.new_type)
            self._properties[-1]['name'] = str(prop.new_name)
        if REQUEST is not None:
            return self.manage_propertiesForm(self, REQUEST)

    def mkpairs(self,seq) :
        return map(mkpair,seq)

    def getReference(self,name) :
        # """        reference """
        return self.lookupReference(getattr(self,name))
        
    def lookupReference(self,ref) :
        # """     """
	if type(ref) is type("") :
            if len(ref) > 6 and ref[0:6] == 'int://' :
                try :
                    ob = self.restrictedTraverse(ref[6:])
                except :
                    return ''
                else :
                    return ob.absolute_url()
        else :
            return ref

default__class_init__ (PropertyManager)

def mkpair(item) :
    if type(item) == type(()) or type(item) == type([]) :
        return item
    else :
        return (item,item)
